#include <eopcc/txt_io.hpp>
#include <iomanip>
#include <string>

namespace eopcc {

const std::string digit_chars = "0123456789e+-.";


TXTReader::TXTReader(const std::string& filename) {
	seed_cloud_.has_color = false;
	seed_cloud_.has_las_metadata = false;
	seed_cloud_.has_time = false;

	infile_ = std::ifstream(filename.c_str());

	// This is a bit slow, but it's simple
	std::string test_str;
	num_points_remaining_ = 0;
	while (std::getline(infile_, test_str))
		num_points_remaining_++;

	// Reset the file to the start
	infile_ = std::ifstream(filename.c_str());
}

PointCloud TXTReader::ReadSubCloud(int sub_cloud_size) {
	int num_to_read = std::min<int>(sub_cloud_size, num_points_remaining_);
	PointCloud point_cloud = seed_cloud_;
	point_cloud.resize(num_to_read);

	for (int i=0; i < num_to_read; ++i) {
		std::string line;
		std::getline(infile_, line);

		size_t idx = line.find_first_not_of(digit_chars);
		point_cloud.loc[i].x = std::stod(line.substr(0, idx));
		line = line.substr(idx+1);

		idx = line.find_first_not_of(digit_chars);
		point_cloud.loc[i].y = std::stod(line.substr(0, idx));
		line = line.substr(idx+1);

		idx = line.find_first_not_of(digit_chars);
		point_cloud.loc[i].z = std::stod(line.substr(0, idx));
		line = line.substr(idx+1);

		idx = line.find_first_not_of(digit_chars);
		if (!line.substr(0, idx).empty()) {
			try {
				point_cloud.intensity[i] = std::stod(line.substr(0, idx));
			} catch (std::invalid_argument& e) {
				point_cloud.intensity[i] = 0;
			}
		} else {
			point_cloud.intensity[i] = 0;
		}
	}

	return point_cloud;
}

TXTWriter::TXTWriter(const std::string& filename) : out_file_(filename) {
}

TXTWriter::~TXTWriter() {}

void TXTWriter::WriteSubCloud(const PointCloud& sub_cloud) {
	for (size_t i=0; i < sub_cloud.size(); ++i) {
		out_file_ << std::fixed << std::setprecision(3)
		          << sub_cloud.loc[i].x << " " << sub_cloud.loc[i].y << " " << sub_cloud.loc[i].z << " " << sub_cloud.intensity[i] << "\n";
	}

	points_written_ += sub_cloud.size();
}

void LoadFromTXT(const std::string& txt_file_name, PointCloud& point_cloud) {
	TXTReader reader(txt_file_name);
	point_cloud = reader.ReadSubCloud(reader.NumRemainingPoints());
}

void WriteToTXT(const std::string& txt_file_name, const PointCloud& point_cloud) {
	TXTWriter writer(txt_file_name);
	writer.WriteSubCloud(point_cloud);
}


}
